package com.example.cs_openmediation_ad

import androidx.annotation.NonNull
import android.app.Activity
import io.flutter.Log;
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import com.openmediation.sdk.InitConfiguration;
import com.openmediation.sdk.OmAds;
import com.openmediation.sdk.InitCallback;
import com.openmediation.sdk.utils.error.Error;
import com.openmediation.sdk.utils.model.Scene;
import com.openmediation.sdk.video.RewardedVideoAd;
import com.openmediation.sdk.video.RewardedVideoListener;

/** CsOpenmediationAdPlugin */
class CsOpenmediationAdPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  /// The MethodChannel that will the communication between Flutter and native Android
  ///
  /// This local reference serves to register the plugin with the Flutter Engine and unregister it
  /// when the Flutter Engine is detached from the Activity
  private lateinit var channel: MethodChannel
  var activity: Activity? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "cs_openmediation_ad")
    channel.setMethodCallHandler(this)
  }

  fun OnMethodCallHandler(method: String, args: Map<String, Any>?) {
    try {
      activity?.runOnUiThread(Runnable { channel.invokeMethod(method!!, args) });
    } catch (e: Exception) {
      Log.e("flutter_ayetstudios", "Error $e");
    }
  }

  fun setupListener() {
    RewardedVideoAd.setAdListener(object:RewardedVideoListener {
      override fun onRewardedVideoAvailabilityChanged(var1: Boolean) {
        var args: Map<String, Any>? = mapOf<String, Any>("available" to var1);
        OnMethodCallHandler("rewardedVideoAvailabilityChanged", args);
      }

      override fun onRewardedVideoAdShowed(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdShowed", args);
      }

      override fun onRewardedVideoAdShowFailed(var1: Scene?, var2: Error?) {
        var args: MutableMap<String, Any>? = mutableMapOf<String, Any>();
        if (var1 != null) {
          args = mutableMapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        if (var2 != null) {
          args!!.put("code", var2.errorCode);
          args!!.put("message", var2.errorMessage);
        }
        OnMethodCallHandler("rewardedVideoAdShowFailed", args);
      }

      override fun onRewardedVideoAdClicked(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdClicked", args);
      }

      override fun onRewardedVideoAdClosed(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdClosed", args);
      }

      override fun onRewardedVideoAdStarted(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdStarted", args);
      }

      override fun onRewardedVideoAdEnded(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdEnded", args);
      }

      override fun onRewardedVideoAdRewarded(var1: Scene?) {
        var args: Map<String, Any>? = null;
        if (var1 != null) {
          args = mapOf<String, Any>("id" to var1.id, "n" to var1.n, "isd" to var1.isd, "oriData" to var1.oriData);
        }
        OnMethodCallHandler("rewardedVideoAdRewarded", args);
      }
    });


  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "init") {
      val appKey: String = call.argument<String>("appKey") ?: return;
      val logEnable: Boolean = call.argument<Boolean>("logEnable") ?: return;
      setupListener();
      val configuration: InitConfiguration = InitConfiguration.Builder()
        .appKey(appKey)
        .preloadAdTypes(OmAds.AD_TYPE.INTERSTITIAL, OmAds.AD_TYPE.REWARDED_VIDEO)
        .logEnable(logEnable)
        .build();
      OmAds.init(configuration, object : InitCallback {
        override fun onSuccess() {
          OnMethodCallHandler("initializationSuccess", null);
        }

        override fun onError(error: Error?) {
          var args: Map<String, Any>? = null;
          if (error != null) {
            args = mapOf<String, Any>("code" to error.errorCode, "message" to error.errorMessage);
          }
          OnMethodCallHandler("initializationFailed", args);
        }
      })
    } else if (call.method == "setUserId") {
      val userId: String = call.argument<String>("userId") ?: return;
      OmAds.setUserId(userId);
    } else if (call.method == "setExtId") {
      val sceneName: String = call.argument<String>("sceneName") ?: return;
      val extId: String = call.argument<String>("extId") ?: return;
      RewardedVideoAd.setExtId(sceneName, extId);
    } else if (call.method == "rewardVideoIsReady") {
      result.success(RewardedVideoAd.isReady());
    } else if (call.method == "showRewardVideo") {
      val sceneName: String? = call.argument<String>("sceneName");
      RewardedVideoAd.showAd(sceneName);
    } else if (call.method == "onResume") {
      OmAds.onResume(activity);
    } else if (call.method == "onPause") {
      OmAds.onPause(activity);
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    channel.setMethodCallHandler(null);
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivityForConfigChanges() {}

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity;
  }

  override fun onDetachedFromActivity() {}
}
